package com.sun.media.sound;

import javax.sound.sampled.FloatControl;
import javax.sound.sampled.BooleanControl;
import javax.sound.sampled.CompoundControl;
import javax.sound.sampled.Control;
import java.util.Vector;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.Line;
import javax.sound.sampled.Port;

final class PortMixer extends AbstractMixer
{
    private static final int SRC_UNKNOWN = 1;
    private static final int SRC_MICROPHONE = 2;
    private static final int SRC_LINE_IN = 3;
    private static final int SRC_COMPACT_DISC = 4;
    private static final int SRC_MASK = 255;
    private static final int DST_UNKNOWN = 256;
    private static final int DST_SPEAKER = 512;
    private static final int DST_HEADPHONE = 768;
    private static final int DST_LINE_OUT = 1024;
    private static final int DST_MASK = 65280;
    private Port.Info[] portInfos;
    private PortMixer.PortMixerPort[] ports;
    private long id;
    
    PortMixer(PortMixerProvider.PortMixerInfo portMixerInfo) {
        super(portMixerInfo, null, null, (Line$Info[])null);
        this.id = 0L;
        int nGetPortCount = 0;
        int n = 0;
        int n2 = 0;
        try {
            try {
                this.id = nOpen(this.getMixerIndex());
                if (this.id != 0L) {
                    nGetPortCount = nGetPortCount(this.id);
                    if (nGetPortCount < 0) {
                        nGetPortCount = 0;
                    }
                }
            }
            catch (Exception ex) {}
            this.portInfos = new Port.Info[nGetPortCount];
            for (int i = 0; i < nGetPortCount; ++i) {
                int nGetPortType = nGetPortType(this.id, i);
                n += ((nGetPortType & 0xFF) != 0x0);
                n2 += ((nGetPortType & 0xFF00) != 0x0);
                this.portInfos[i] = this.getPortInfo(i, nGetPortType);
            }
        }
        finally {
            if (this.id != 0L) {
                nClose(this.id);
            }
            this.id = 0L;
        }
        this.sourceLineInfo = new Port.Info[n];
        this.targetLineInfo = new Port.Info[n2];
        int n3 = 0;
        int n4 = 0;
        for (int j = 0; j < nGetPortCount; ++j) {
            if (this.portInfos[j].isSource()) {
                this.sourceLineInfo[n3++] = this.portInfos[j];
            }
            else {
                this.targetLineInfo[n4++] = this.portInfos[j];
            }
        }
    }
    
    @Override
    public Line getLine(Line.Info obj) throws LineUnavailableException {
        Line.Info lineInfo = this.getLineInfo(obj);
        if (lineInfo != null && lineInfo instanceof Port.Info) {
            int i = 0;
            while (i < this.portInfos.length) {
                if (lineInfo.equals(this.portInfos[i])) {
                    return this.getPort(i);
                }
                ++i;
            }
        }
        throw new IllegalArgumentException("Line unsupported: " + obj);
    }
    
    @Override
    public int getMaxLines(Line.Info info) {
        Line.Info lineInfo = this.getLineInfo(info);
        if (lineInfo == null) {
            return 0;
        }
        if ((lineInfo instanceof Port.Info)) {
            return 1;
        }
        return 0;
    }
    
    @Override
    protected void implOpen() throws LineUnavailableException {
        this.id = nOpen(this.getMixerIndex());
    }
    
    @Override
    protected void implClose() {
        long id = this.id;
        this.id = 0L;
        nClose(id);
        if (this.ports != null) {
            for (int i = 0; i < this.ports.length; ++i) {
                if (this.ports[i] != null) {
                    PortMixer.PortMixerPort.access$000(this.ports[i]);
                }
            }
        }
    }
    
    @Override
    protected void implStart() {
    }
    
    @Override
    protected void implStop() {
    }
    
    private Port.Info getPortInfo(int n, int n2) {
        switch (n2) {
            case 1: {
                return new PortMixer.PortInfo(nGetPortName(this.getID(), n), true);
            }
            case 2: {
                return Port.Info.MICROPHONE;
            }
            case 3: {
                return Port.Info.LINE_IN;
            }
            case 4: {
                return Port.Info.COMPACT_DISC;
            }
            case 256: {
                return new PortMixer.PortInfo(nGetPortName(this.getID(), n), false);
            }
            case 512: {
                return Port.Info.SPEAKER;
            }
            case 768: {
                return Port.Info.HEADPHONE;
            }
            case 1024: {
                return Port.Info.LINE_OUT;
            }
            default: {
                return null;
            }
        }
    }
    
    int getMixerIndex() {
        return ((PortMixerProvider.PortMixerInfo)this.getMixerInfo()).getIndex();
    }
    
    Port getPort(int n) {
        if (this.ports == null) {
            this.ports = new PortMixer.PortMixerPort[this.portInfos.length];
        }
        if (this.ports[n] == null) {
            return this.ports[n] = new PortMixer.PortMixerPort(this.portInfos[n], this, n);
        }
        return this.ports[n];
    }
    
    long getID() {
        return this.id;
    }
    
    private static native long nOpen(int p0) throws LineUnavailableException;
    
    private static native void nClose(long p0);
    
    private static native int nGetPortCount(long p0);
    
    private static native int nGetPortType(long p0, int p1);
    
    private static native String nGetPortName(long p0, int p1);
    
    private static native void nGetControls(long p0, int p1, Vector p2);
    
    private static native void nControlSetIntValue(long p0, int p1);
    
    private static native int nControlGetIntValue(long p0);
    
    private static native void nControlSetFloatValue(long p0, float p1);
    
    private static native float nControlGetFloatValue(long p0);
    
    static void access$300(long n, int n2, Vector vector) {
        nGetControls(n, n2, vector);
    }
    
    static void access$700(long n, int n2) {
        nControlSetIntValue(n, n2);
    }
    
    static int access$800(long n) {
        return nControlGetIntValue(n);
    }
    
    static void access$1100(long n, float n2) {
        nControlSetFloatValue(n, n2);
    }
    
    static float access$1200(long n) {
        return nControlGetFloatValue(n);
    }
    
    private static final class PortMixerPort extends AbstractLine implements Port
    {
        private final int portIndex;
        private long id;
        
        private PortMixerPort(Port.Info info, PortMixer portMixer, int portIndex) {
            super(info, portMixer, (Control[])null);
            this.portIndex = portIndex;
        }
        
        void implOpen() throws LineUnavailableException {
            long id = ((PortMixer)this.mixer).getID();
            if (this.id == 0L || id != this.id || this.controls.length == 0) {
                this.id = id;
                Vector<Object> vector = new Vector();
                synchronized (vector) {
                    PortMixer.access$300(this.id, this.portIndex, vector);
                    this.controls = new Control[vector.size()];
                    for (int i = 0; i < this.controls.length; ++i) {
                        this.controls[i] = (Control)vector.elementAt(i);
                    }
                }
            }
            else {
                this.enableControls(this.controls, true);
            }
        }
        
        private void enableControls(Control[] array, boolean b) {
            for (int i = 0; i < array.length; ++i) {
                if ((array[i] instanceof PortMixer.BoolCtrl)) {
                    ((PortMixer.BoolCtrl)array[i]).closed = !b;
                }
                else if ((array[i] instanceof PortMixer.FloatCtrl)) {
                    ((PortMixer.FloatCtrl)array[i]).closed = !b;
                }
                else if ((array[i] instanceof CompoundControl)) {
                    this.enableControls(((CompoundControl)array[i]).getMemberControls(), b);
                }
            }
        }
        
        private void disposeControls() {
            this.enableControls(this.controls, false);
            this.controls = new Control[0];
        }
        
        void implClose() {
            this.enableControls(this.controls, false);
        }
        
        @Override
        public void open() throws LineUnavailableException {
            synchronized (this.mixer) {
                if (!this.isOpen()) {
                    this.mixer.open(this);
                    try {
                        this.implOpen();
                        this.setOpen(true);
                    }
                    catch (LineUnavailableException ex) {
                        this.mixer.close(this);
                        throw ex;
                    }
                }
            }
        }
        
        @Override
        public void close() {
            synchronized (this.mixer) {
                if (this.isOpen()) {
                    this.setOpen(false);
                    this.implClose();
                    this.mixer.close(this);
                }
            }
        }
        
        static void access$000(PortMixer.PortMixerPort portMixerPort) {
            portMixerPort.disposeControls();
        }
    }
    
    private static final class BoolCtrl extends BooleanControl
    {
        private final long controlID;
        private boolean closed;
        
        private static BooleanControl.Type createType(String s) {
            if (s.equals("Mute")) {
                return BooleanControl.Type.MUTE;
            }
            s.equals("Select");
            return new BCT(s);
        }
        
        private BoolCtrl(long n, String s) {
            this(n, createType(s));
        }
        
        private BoolCtrl(long controlID, BooleanControl.Type type) {
            super(type, false);
            this.closed = false;
            this.controlID = controlID;
        }
        
        @Override
        public void setValue(boolean b) {
            if (!this.closed) {
                PortMixer.access$700(this.controlID, b ? 1 : 0);
            }
        }
        
        @Override
        public boolean getValue() {
            return !this.closed && PortMixer.access$800(this.controlID) != 0;
        }
        
        private static final class BCT extends BooleanControl.Type
        {
            private BCT(String name) {
                super(name);
            }
        }
    }
    
    private static final class CompCtrl extends CompoundControl
    {
        private CompCtrl(String s, Control[] memberControls) {
            super(new CCT(s), memberControls);
        }
        
        private static final class CCT extends CompoundControl.Type
        {
            private CCT(String name) {
                super(name);
            }
        }
    }
    
    private static final class FloatCtrl extends FloatControl
    {
        private final long controlID;
        private boolean closed;
        private static final FloatControl.Type[] FLOAT_CONTROL_TYPES;
        
        private FloatCtrl(long n, String s, float n2, float n3, float n4, String s2) {
            this(n, new FCT(s), n2, n3, n4, s2);
        }
        
        private FloatCtrl(long n, int n2, float n3, float n4, float n5, String s) {
            this(n, PortMixer.FloatCtrl.FLOAT_CONTROL_TYPES[n2], n3, n4, n5, s);
        }
        
        private FloatCtrl(long controlID, FloatControl.Type type, float n, float maximum, float precision, String units) {
            super(type, n, maximum, precision, 1000, n, units);
            this.closed = false;
            this.controlID = controlID;
        }
        
        @Override
        public void setValue(float n) {
            if (!this.closed) {
                PortMixer.access$1100(this.controlID, n);
            }
        }
        
        @Override
        public float getValue() {
            if (!this.closed) {
                return PortMixer.access$1200(this.controlID);
            }
            return this.getMinimum();
        }
        
        static {
            FLOAT_CONTROL_TYPES = new FloatControl.Type[] { null, FloatControl.Type.BALANCE, FloatControl.Type.MASTER_GAIN, FloatControl.Type.PAN, FloatControl.Type.VOLUME };
        }
        
        private static final class FCT extends FloatControl.Type
        {
            private FCT(String name) {
                super(name);
            }
        }
    }
    
    private static final class PortInfo extends Port.Info
    {
        private PortInfo(String name, boolean isSource) {
            super(Port.class, name, isSource);
        }
    }
}
